//---------------------------------------------------------------------------
#ifndef PgpSdkH
#define PgpSdkH
#include <forms.hpp>
#pragma hdrstop

class PACKAGE TPGPsdk : public TObject
{
public:
   TForm *Owner;
   TStringList *headers;
   AnsiString mainpubringpkr, mainsecringskr; // main keyrings
   AnsiString pubringpkr, secringskr, remringpkr, initerrormsg; // QS keyrings
   unsigned err, apiversion, sdkversion;
   AnsiString preferredalgo;
   bool InitSuccess;

   virtual __fastcall TPGPsdk();
   virtual __fastcall TPGPsdk(TForm *Owner, const AnsiString &pubring, const AnsiString &secring,
      const AnsiString &remring);
   virtual __fastcall~TPGPsdk();

   AnsiString __fastcall GetEncryptKey(int beforeEncodeLength);

   virtual bool __fastcall ExportDefaultKeys(TStrings *exports);
   virtual bool __fastcall ExportDefaultKeysFast(TStrings *exports);
   virtual bool __fastcall UpdateQSKeyrings(TControl *ForHourGlass);

   virtual bool __fastcall VerifyClearSigned(TStrings *SignedText);
   virtual bool __fastcall DecryptMessage(TStrings *Message, char *Passphrase, TStrings *DecKeyIds, bool recurse);
   virtual bool __fastcall SignMessage(TStrings *Message, char *keyid, char *Passphrase);
   virtual bool __fastcall EncryptMessage(TStrings *Message, TStrings *KeyIds);
   virtual bool __fastcall EncryptSignMessage(TStrings *Message, TStrings *EncKeyIds, char *SignKeyid, char *Passphrase);
   virtual bool __fastcall CEncryptMessage(TStrings *Message, const AnsiString &Passphrase);

   virtual bool __fastcall EncryptToUserID(const AnsiString &Pubring, const AnsiString &UserID, TStrings *Text);
   virtual bool __fastcall EncryptToUserIDs(const AnsiString &Pubring, TStrings *UserIDs, TStrings *Text);
   virtual bool __fastcall EncryptTo(const AnsiString &Pubring, const AnsiString &KeyID, TStrings *Text);
   virtual bool __fastcall EncryptTo(const AnsiString &Pubring, TStrings *KeyIDs, TStrings *Text);

   virtual bool __fastcall CheckPGP6NeedsUpdate(TControl *ForHourGlass);

   virtual bool __fastcall GetKeysLV(const AnsiString &pubRing, const AnsiString &secRing, TStrings *Lines);
   virtual bool __fastcall GetKeyLV(const AnsiString &pubring, const AnsiString &secring,
      const AnsiString &keyid, AnsiString &keydesc);
   virtual bool __fastcall GetKeys(const AnsiString &pubRing, const AnsiString &secRing, TStrings *Lines);
   virtual bool __fastcall GetKeys(const AnsiString &pubRing, TStrings *Lines);
   virtual AnsiString __fastcall GetKeyID(const AnsiString &Pubring, const AnsiString &UserID);

   virtual bool __fastcall ExportKeys(const AnsiString Pubring, const AnsiString Secring, TStrings *KeyIDs,
      bool IncludePrivates, TStrings *Exports);
   virtual bool __fastcall ExportKey(const AnsiString Pubring, const AnsiString Secring, AnsiString KeyID,
      bool IncludePrivate, TStrings *Export);

   virtual bool __fastcall RandomMouseMoved();
   virtual bool __fastcall GetRandomBytes(void *dataBuf, int size);

   virtual AnsiString __fastcall MessageDecryptVerifyLine(bool first, int decs, int sigs);
   virtual void __fastcall PushHeaders(TStrings *message);
   virtual void __fastcall PopHeaders(TStrings *message);
   virtual AnsiString __fastcall GetLastErrorStr();
   virtual int  __fastcall IdNo();
};

PACKAGE TPGPsdk* __fastcall PGPInit(TForm *Owner);
PACKAGE TPGPsdk* __fastcall PGPInitV6(TForm *Owner, const AnsiString &pubring,
   const AnsiString &secring, const AnsiString &remring);
PACKAGE TPGPsdk* __fastcall PGPInitV8(TForm *Owner, const AnsiString &pubring,
   const AnsiString &secring, const AnsiString &remring);

PACKAGE AnsiString __fastcall PGPPublicRing();
PACKAGE AnsiString __fastcall PGPSecretRing();
PACKAGE AnsiString __fastcall PGPRemailerRing();
PACKAGE AnsiString __fastcall PGPNymserverRing();
// To get keyring names if none specified
PACKAGE bool __fastcall GetPGPkeyrings(TForm *Owner, AnsiString &PublicRing, AnsiString &SecretRing);

extern PACKAGE TPGPsdk *PGP;
extern TPGPsdk *PGPsdk;
extern AnsiString MixmasterDir;

const AnsiString BeginPgpPubKeyBlock = "-----BEGIN PGP PUBLIC KEY BLOCK-----";
const AnsiString EndPgpPubKeyBlock = "-----END PGP PUBLIC KEY BLOCK-----";
const AnsiString BeginPgpPriKeyBlock = "-----BEGIN PGP PRIVATE KEY BLOCK-----";
const AnsiString EndPgpPriKeyBlock = "-----END PGP PRIVATE KEY BLOCK-----";
const AnsiString BeginPgpSecKeyBlock = "-----BEGIN PGP SECRET KEY BLOCK-----";
const AnsiString EndPgpSecKeyBlock = "-----END PGP SECRET KEY BLOCK-----";
const AnsiString BeginPgpMessage = "-----BEGIN PGP MESSAGE-----";
const AnsiString EndPgpMessage = "-----END PGP MESSAGE-----";
const AnsiString BeginPgpSignedMessage = "-----BEGIN PGP SIGNED MESSAGE-----";
const AnsiString BeginPgpSignature = "-----BEGIN PGP SIGNATURE-----";
const AnsiString EndPgpSignature = "-----END PGP SIGNATURE-----";

// listview key fields
const int lvUserId  = 0;
const int lvKeyID   = 1;
const int lvStatus  = 2; // validity, revoked, expired, disabled
const int lvBits    = 3;
const int lvAlgo    = 4;
const int lvADKs    = 5;
const int lvSecret  = 6;
const int lvVersion = 7;

// key size for GetEncryptKey
const int ekLength = 24;

//---------------------------------------------------------------------------
#endif
